#pragma once

#include <string>

#include "envoy/config/filter/http/transcoder/v2/transcoder.pb.h"
#include "envoy/server/instance.h"

#include "common/config/well_known_names.h"

#include "server/config/network/http_connection_manager.h"

namespace Envoy {
namespace Server {
namespace Configuration {

/**
 * Config registration for the gRPC JSON transcoder filter. @see NamedHttpFilterConfigFactory.
 */
class GrpcJsonTranscoderFilterConfig : public NamedHttpFilterConfigFactory {
public:
  HttpFilterFactoryCb createFilterFactory(const Json::Object& json_config,
                                          const std::string& stats_prefix,
                                          FactoryContext& context) override;
  HttpFilterFactoryCb createFilterFactoryFromProto(const Protobuf::Message& proto_config,
                                                   const std::string& stats_prefix,
                                                   FactoryContext& context) override;

  ProtobufTypes::MessagePtr createEmptyConfigProto() override {
    return ProtobufTypes::MessagePtr{
        new envoy::config::filter::http::transcoder::v2::GrpcJsonTranscoder()};
  }

  std::string name() override { return Config::HttpFilterNames::get().GRPC_JSON_TRANSCODER; };

private:
  HttpFilterFactoryCb
  createFilter(const envoy::config::filter::http::transcoder::v2::GrpcJsonTranscoder& proto_config,
               const std::string& stats_prefix, FactoryContext& context);
};

} // namespace Configuration
} // namespace Server
} // namespace Envoy
